
# parsetab.py
# This file is automatically generated. Do not edit.
# pylint: disable=W,C,R
_tabversion = '3.10'

_lr_method = 'LALR'

_lr_signature = 'BEGIN_PARENTHESIS BRACKET_ARGUMENT_CONTENT BRACKET_ARGUMENT_END BRACKET_ARGUMENT_START COMMAND_INVOCATION_END COMMAND_INVOCATION_START END_PARENTHESIS LINE_COMMENT NEWLINES QUOTED_ARGUMENT_CONTENT QUOTED_ARGUMENT_END QUOTED_ARGUMENT_START SPACES UNQUOTED_ARGUMENTfile : file file_element\n                | file_element\n                | emptyfile_element : line_ending\n                        | spaces\n                        | command_invocationcommand_invocation : start_cmd_invoke arguments end_cmd_invokeparentheses : start_parenthesis arguments end_parenthesisarguments : arguments argument\n                     | argumentargument : bracket_argument\n                    | quoted_argument\n                    | unquoted_argument\n                    | arguments_separation\n                    | parentheses\n                    | emptyarguments_separation : spaces\n                                | line_endingquoted_argument : QUOTED_ARGUMENT_START quoted_argument_content QUOTED_ARGUMENT_ENDbracket_argument : BRACKET_ARGUMENT_START bracket_argument_content BRACKET_ARGUMENT_ENDline_ending : line_comment newlines\n                       | newlinesunquoted_argument : UNQUOTED_ARGUMENTline_comment : LINE_COMMENTnewlines : NEWLINESempty :bracket_argument_content : bracket_argument_content bracket_argument_content_element\n                                    | bracket_argument_content_elementquoted_argument_content : quoted_argument_content quoted_argument_content_element\n                                   | quoted_argument_content_element\n                                   | emptybracket_argument_content_element : BRACKET_ARGUMENT_CONTENTquoted_argument_content_element : QUOTED_ARGUMENT_CONTENTspaces : SPACESstart_parenthesis : BEGIN_PARENTHESISend_parenthesis : END_PARENTHESISstart_cmd_invoke : COMMAND_INVOCATION_STARTend_cmd_invoke : COMMAND_INVOCATION_END'
    
_lr_action_items = {'SPACES':([0,1,2,3,4,5,6,8,9,10,12,13,14,15,16,17,18,19,20,21,22,23,26,27,28,29,30,31,32,33,41,42,44,46,47,],[9,9,-2,-3,-4,-5,-6,-22,-34,9,-25,-37,-1,-21,9,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,9,-35,-7,-9,-38,9,-20,-19,-8,-36,]),'LINE_COMMENT':([0,1,2,3,4,5,6,8,9,10,12,13,14,15,16,17,18,19,20,21,22,23,26,27,28,29,30,31,32,33,41,42,44,46,47,],[11,11,-2,-3,-4,-5,-6,-22,-34,11,-25,-37,-1,-21,11,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,11,-35,-7,-9,-38,11,-20,-19,-8,-36,]),'NEWLINES':([0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,26,27,28,29,30,31,32,33,41,42,44,46,47,],[12,12,-2,-3,-4,-5,-6,12,-22,-34,12,-24,-25,-37,-1,-21,12,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,12,-35,-7,-9,-38,12,-20,-19,-8,-36,]),'COMMAND_INVOCATION_START':([0,1,2,3,4,5,6,8,9,12,14,15,31,33,],[13,13,-2,-3,-4,-5,-6,-22,-34,-25,-1,-21,-7,-38,]),'$end':([0,1,2,3,4,5,6,8,9,12,14,15,31,33,],[-26,0,-2,-3,-4,-5,-6,-22,-34,-25,-1,-21,-7,-38,]),'COMMAND_INVOCATION_END':([8,9,10,12,13,15,16,17,18,19,20,21,22,23,26,27,28,32,42,44,46,47,],[-22,-34,-26,-25,-37,-21,33,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,-9,-20,-19,-8,-36,]),'BRACKET_ARGUMENT_START':([8,9,10,12,13,15,16,17,18,19,20,21,22,23,26,27,28,29,30,32,41,42,44,46,47,],[-22,-34,24,-25,-37,-21,24,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,24,-35,-9,24,-20,-19,-8,-36,]),'QUOTED_ARGUMENT_START':([8,9,10,12,13,15,16,17,18,19,20,21,22,23,26,27,28,29,30,32,41,42,44,46,47,],[-22,-34,25,-25,-37,-21,25,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,25,-35,-9,25,-20,-19,-8,-36,]),'UNQUOTED_ARGUMENT':([8,9,10,12,13,15,16,17,18,19,20,21,22,23,26,27,28,29,30,32,41,42,44,46,47,],[-22,-34,26,-25,-37,-21,26,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,26,-35,-9,26,-20,-19,-8,-36,]),'BEGIN_PARENTHESIS':([8,9,10,12,13,15,16,17,18,19,20,21,22,23,26,27,28,29,30,32,41,42,44,46,47,],[-22,-34,30,-25,-37,-21,30,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,30,-35,-9,30,-20,-19,-8,-36,]),'END_PARENTHESIS':([8,9,12,15,17,18,19,20,21,22,23,26,27,28,29,30,32,41,42,44,46,47,],[-22,-34,-25,-21,-10,-11,-12,-13,-14,-15,-16,-23,-17,-18,-26,-35,-9,47,-20,-19,-8,-36,]),'BRACKET_ARGUMENT_CONTENT':([24,34,35,36,43,],[36,36,-28,-32,-27,]),'QUOTED_ARGUMENT_CONTENT':([25,37,38,39,40,45,],[40,40,-30,-31,-33,-29,]),'QUOTED_ARGUMENT_END':([25,37,38,39,40,45,],[-26,44,-30,-31,-33,-29,]),'BRACKET_ARGUMENT_END':([34,35,36,43,],[42,-28,-32,-27,]),}

_lr_action = {}
for _k, _v in _lr_action_items.items():
   for _x,_y in zip(_v[0],_v[1]):
      if not _x in _lr_action:  _lr_action[_x] = {}
      _lr_action[_x][_k] = _y
del _lr_action_items

_lr_goto_items = {'file':([0,],[1,]),'file_element':([0,1,],[2,14,]),'empty':([0,10,16,25,29,41,],[3,23,23,39,23,23,]),'line_ending':([0,1,10,16,29,41,],[4,4,28,28,28,28,]),'spaces':([0,1,10,16,29,41,],[5,5,27,27,27,27,]),'command_invocation':([0,1,],[6,6,]),'line_comment':([0,1,10,16,29,41,],[7,7,7,7,7,7,]),'newlines':([0,1,7,10,16,29,41,],[8,8,15,8,8,8,8,]),'start_cmd_invoke':([0,1,],[10,10,]),'arguments':([10,29,],[16,41,]),'argument':([10,16,29,41,],[17,32,17,32,]),'bracket_argument':([10,16,29,41,],[18,18,18,18,]),'quoted_argument':([10,16,29,41,],[19,19,19,19,]),'unquoted_argument':([10,16,29,41,],[20,20,20,20,]),'arguments_separation':([10,16,29,41,],[21,21,21,21,]),'parentheses':([10,16,29,41,],[22,22,22,22,]),'start_parenthesis':([10,16,29,41,],[29,29,29,29,]),'end_cmd_invoke':([16,],[31,]),'bracket_argument_content':([24,],[34,]),'bracket_argument_content_element':([24,34,],[35,43,]),'quoted_argument_content':([25,],[37,]),'quoted_argument_content_element':([25,37,],[38,45,]),'end_parenthesis':([41,],[46,]),}

_lr_goto = {}
for _k, _v in _lr_goto_items.items():
   for _x, _y in zip(_v[0], _v[1]):
       if not _x in _lr_goto: _lr_goto[_x] = {}
       _lr_goto[_x][_k] = _y
del _lr_goto_items
_lr_productions = [
  ("S' -> file","S'",1,None,None,None),
  ('file -> file file_element','file',2,'p_file','cmake_parser.py',25),
  ('file -> file_element','file',1,'p_file','cmake_parser.py',26),
  ('file -> empty','file',1,'p_file','cmake_parser.py',27),
  ('file_element -> line_ending','file_element',1,'p_file_element','cmake_parser.py',35),
  ('file_element -> spaces','file_element',1,'p_file_element','cmake_parser.py',36),
  ('file_element -> command_invocation','file_element',1,'p_file_element','cmake_parser.py',37),
  ('command_invocation -> start_cmd_invoke arguments end_cmd_invoke','command_invocation',3,'p_command_invocation','cmake_parser.py',42),
  ('parentheses -> start_parenthesis arguments end_parenthesis','parentheses',3,'p_parentheses','cmake_parser.py',49),
  ('arguments -> arguments argument','arguments',2,'p_arguments','cmake_parser.py',56),
  ('arguments -> argument','arguments',1,'p_arguments','cmake_parser.py',57),
  ('argument -> bracket_argument','argument',1,'p_argument','cmake_parser.py',65),
  ('argument -> quoted_argument','argument',1,'p_argument','cmake_parser.py',66),
  ('argument -> unquoted_argument','argument',1,'p_argument','cmake_parser.py',67),
  ('argument -> arguments_separation','argument',1,'p_argument','cmake_parser.py',68),
  ('argument -> parentheses','argument',1,'p_argument','cmake_parser.py',69),
  ('argument -> empty','argument',1,'p_argument','cmake_parser.py',70),
  ('arguments_separation -> spaces','arguments_separation',1,'p_arguments_separation','cmake_parser.py',75),
  ('arguments_separation -> line_ending','arguments_separation',1,'p_arguments_separation','cmake_parser.py',76),
  ('quoted_argument -> QUOTED_ARGUMENT_START quoted_argument_content QUOTED_ARGUMENT_END','quoted_argument',3,'p_quoted_argument','cmake_parser.py',81),
  ('bracket_argument -> BRACKET_ARGUMENT_START bracket_argument_content BRACKET_ARGUMENT_END','bracket_argument',3,'p_bracket_argument','cmake_parser.py',86),
  ('line_ending -> line_comment newlines','line_ending',2,'p_line_ending','cmake_parser.py',94),
  ('line_ending -> newlines','line_ending',1,'p_line_ending','cmake_parser.py',95),
  ('unquoted_argument -> UNQUOTED_ARGUMENT','unquoted_argument',1,'p_unquoted_argument','cmake_parser.py',102),
  ('line_comment -> LINE_COMMENT','line_comment',1,'p_line_comment','cmake_parser.py',107),
  ('newlines -> NEWLINES','newlines',1,'p_newlines','cmake_parser.py',112),
  ('empty -> <empty>','empty',0,'p_empty','cmake_parser.py',118),
  ('bracket_argument_content -> bracket_argument_content bracket_argument_content_element','bracket_argument_content',2,'p_bracket_argument_content','cmake_parser.py',123),
  ('bracket_argument_content -> bracket_argument_content_element','bracket_argument_content',1,'p_bracket_argument_content','cmake_parser.py',124),
  ('quoted_argument_content -> quoted_argument_content quoted_argument_content_element','quoted_argument_content',2,'p_quoted_argument_content','cmake_parser.py',129),
  ('quoted_argument_content -> quoted_argument_content_element','quoted_argument_content',1,'p_quoted_argument_content','cmake_parser.py',130),
  ('quoted_argument_content -> empty','quoted_argument_content',1,'p_quoted_argument_content','cmake_parser.py',131),
  ('bracket_argument_content_element -> BRACKET_ARGUMENT_CONTENT','bracket_argument_content_element',1,'p_bracket_argument_content_element','cmake_parser.py',136),
  ('quoted_argument_content_element -> QUOTED_ARGUMENT_CONTENT','quoted_argument_content_element',1,'p_quoted_argument_content_element','cmake_parser.py',141),
  ('spaces -> SPACES','spaces',1,'p_spaces','cmake_parser.py',146),
  ('start_parenthesis -> BEGIN_PARENTHESIS','start_parenthesis',1,'p_start_parenthesis','cmake_parser.py',151),
  ('end_parenthesis -> END_PARENTHESIS','end_parenthesis',1,'p_end_parenthesis','cmake_parser.py',156),
  ('start_cmd_invoke -> COMMAND_INVOCATION_START','start_cmd_invoke',1,'p_start_cmd_invoke','cmake_parser.py',161),
  ('end_cmd_invoke -> COMMAND_INVOCATION_END','end_cmd_invoke',1,'p_end_cmd_invoke','cmake_parser.py',166),
]
